package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.DictConstants;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.domain.entity.SysDictData;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.ModelImageComment;
import com.mcwl.resource.domain.dto.ModelImageCommentRes;
import com.mcwl.resource.domain.dto.ModelImagePageRes;
import com.mcwl.resource.domain.dto.ModelImageRes;
import com.mcwl.resource.domain.vo.ModelImageCommentVo;
import com.mcwl.resource.domain.vo.ModelImageVo;
import com.mcwl.resource.mapper.ModelImageCommentMapper;
import com.mcwl.resource.mapper.ModelImageMapper;
import com.mcwl.resource.service.ModelImageService;
import com.mcwl.system.init.DictInit;
import com.mcwl.system.service.ISysDictDataService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class ModelImageServiceImpl extends ServiceImpl<ModelImageMapper, ModelImage> implements ModelImageService {

    private final ModelImageCommentMapper modelImageCommentMapper;

    private final ModelImageMapper modelImageMapper;

    private final ISysUserService sysUserService;

    private final ISysDictDataService sysDictDataService;



    @Override
    public void updateById(ModelImageRes modelImageRes) {
        if (Objects.isNull(modelImageRes.getId())) {
            return;
        }
        ModelImage modelImage = new ModelImage();
        BeanUtil.copyProperties(modelImageRes, modelImage);
        modelImageMapper.updateById(modelImage);
    }

    @Override
    public void publish(ModelImageRes modelImageRes) {

        ModelImage modelImage = new ModelImage();
        BeanUtil.copyProperties(modelImageRes, modelImage);
        modelImage.setUserId(SecurityUtils.getUserId());
        modelImage.setStatus(3);
        modelImageMapper.insert(modelImage);


    }





    /**
     * 分页查询
     *
     * @param imagePageRes 分页参数
     * @return 分页数据
     */
    @Override
    public TableDataInfo listByPage(ModelImagePageRes imagePageRes) {

        Page<ModelImage> page = new Page<>(imagePageRes.getPageNum(), imagePageRes.getPageSize());
        if (StringUtils.isEmpty(imagePageRes.getOrderByColumn())) {
            imagePageRes.setOrderByColumn("create_time");
        }
        // 设置排序
        boolean isAsc = Objects.equals(imagePageRes.getIsAsc(), "asc");
        OrderItem orderItem = new OrderItem(imagePageRes.getOrderByColumn(), isAsc);
        page.addOrder(orderItem);

        LambdaQueryWrapper<ModelImage> lqw = new LambdaQueryWrapper<>();
        lqw.eq(imagePageRes.getStatus() != null, ModelImage::getStatus, imagePageRes.getStatus())
                .eq(imagePageRes.getUserId() != null, ModelImage::getUserId, imagePageRes.getUserId())
                .ge(imagePageRes.getStartTime() != null, ModelImage::getCreateTime, imagePageRes.getStartTime())
                .le(imagePageRes.getEndTime() != null, ModelImage::getCreateTime, imagePageRes.getEndTime());

        modelImageMapper.selectPage(page, lqw);
        // 获取分页数据
        List<ModelImage> modelImageList = page.getRecords();

        // ModelImage数据转为ModelImagePageVo
        List<ModelImageVo> modelImageVoList = new ArrayList<>();
        for (ModelImage modelImage : modelImageList) {
            ModelImageVo modelImageVo = new ModelImageVo();
            BeanUtil.copyProperties(modelImage, modelImageVo);
            // 获取用户信息
            SysUser sysUser = sysUserService.selectUserById(modelImage.getUserId());
            modelImageVo.setUserId(sysUser.getUserId());
            modelImageVo.setUserName(sysUser.getUserName());
            modelImageVo.setUserAvatar(sysUser.getAvatar());
            modelImageVoList.add(modelImageVo);
        }

        // 封装分页数据
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(modelImageVoList);
        rspData.setTotal(page.getTotal());

        return rspData;
    }

    @Override
    public ModelImageVo getDetail(Long imageId) {

        ModelImageVo modelImageVo = new ModelImageVo();
        ModelImage modelImage = modelImageMapper.selectById(imageId);
        if (Objects.isNull(modelImage)) {
            return null;
        }
        BeanUtil.copyProperties(modelImage, modelImageVo);

        // 设置标签
        List<String> tagList = new ArrayList<>();
        if (StringUtils.isNotEmpty(modelImage.getTags())) {
            String[] tagValueList = modelImage.getTags().split(",");
            for (String tagValue : tagValueList) {
                String dictLabel = DictInit.getDictValue(DictConstants.IMAGE_LABEL, tagValue);
                tagList.add(dictLabel);
            }
        }
        modelImageVo.setTags(tagList);

        SysUser sysUser = sysUserService.selectUserById(modelImage.getUserId());
        modelImageVo.setUserId(SecurityUtils.getUserId());
        modelImageVo.setUserName(SecurityUtils.getUsername());
        modelImageVo.setUserAvatar(sysUser.getAvatar());
        return modelImageVo;
    }



}
